@@ -33,18 +33,18 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/clock"
 
-	velerov1api "github.com/heptio/velero/pkg/apis/velero/v1"
-	pkgbackup "github.com/heptio/velero/pkg/backup"
-	"github.com/heptio/velero/pkg/builder"
-	"github.com/heptio/velero/pkg/generated/clientset/versioned/fake"
-	informers "github.com/heptio/velero/pkg/generated/informers/externalversions"
-	"github.com/heptio/velero/pkg/metrics"
-	"github.com/heptio/velero/pkg/persistence"
-	persistencemocks "github.com/heptio/velero/pkg/persistence/mocks"
-	"github.com/heptio/velero/pkg/plugin/clientmgmt"
-	pluginmocks "github.com/heptio/velero/pkg/plugin/mocks"
-	"github.com/heptio/velero/pkg/plugin/velero"
-	"github.com/heptio/velero/pkg/util/logging"
+	velerov1api "github.com/vmware-tanzu/velero/pkg/apis/velero/v1"
+	pkgbackup "github.com/vmware-tanzu/velero/pkg/backup"
+	"github.com/vmware-tanzu/velero/pkg/builder"
+	"github.com/vmware-tanzu/velero/pkg/generated/clientset/versioned/fake"
+	informers "github.com/vmware-tanzu/velero/pkg/generated/informers/externalversions"
+	"github.com/vmware-tanzu/velero/pkg/metrics"
+	"github.com/vmware-tanzu/velero/pkg/persistence"
+	persistencemocks "github.com/vmware-tanzu/velero/pkg/persistence/mocks"
+	"github.com/vmware-tanzu/velero/pkg/plugin/clientmgmt"
+	pluginmocks "github.com/vmware-tanzu/velero/pkg/plugin/mocks"
+	"github.com/vmware-tanzu/velero/pkg/plugin/velero"
+	"github.com/vmware-tanzu/velero/pkg/util/logging"
 )
 
 type fakeBackupper struct {
@@ -305,7 +305,7 @@ func TestDefaultBackupTTL(t *testing.T) {
 			res := c.prepareBackupRequest(test.backup)
 			assert.NotNil(t, res)
 			assert.Equal(t, test.expectedTTL, res.Spec.TTL)
-			assert.Equal(t, test.expectedExpiration, res.Status.Expiration)
+			assert.Equal(t, test.expectedExpiration, *res.Status.Expiration)
 		})
 	}
 }
@@ -316,6 +316,7 @@ func TestProcessBackupCompletions(t *testing.T) {
 	now, err := time.Parse(time.RFC1123Z, time.RFC1123Z)
 	require.NoError(t, err)
 	now = now.Local()
+	timestamp := metav1.NewTime(now)
 
 	tests := []struct {
 		name                string
@@ -348,9 +349,9 @@ func TestProcessBackupCompletions(t *testing.T) {
 				Status: velerov1api.BackupStatus{
 					Phase:               velerov1api.BackupPhaseCompleted,
 					Version:             1,
-					StartTimestamp:      metav1.NewTime(now),
-					CompletionTimestamp: metav1.NewTime(now),
-					Expiration:          metav1.NewTime(now),
+					StartTimestamp:      &timestamp,
+					CompletionTimestamp: &timestamp,
+					Expiration:          &timestamp,
 				},
 			},
 		},
@@ -376,9 +377,9 @@ func TestProcessBackupCompletions(t *testing.T) {
 				Status: velerov1api.BackupStatus{
 					Phase:               velerov1api.BackupPhaseCompleted,
 					Version:             1,
-					StartTimestamp:      metav1.NewTime(now),
-					CompletionTimestamp: metav1.NewTime(now),
-					Expiration:          metav1.NewTime(now),
+					StartTimestamp:      &timestamp,
+					CompletionTimestamp: &timestamp,
+					Expiration:          &timestamp,
 				},
 			},
 		},
@@ -407,9 +408,9 @@ func TestProcessBackupCompletions(t *testing.T) {
 				Status: velerov1api.BackupStatus{
 					Phase:               velerov1api.BackupPhaseCompleted,
 					Version:             1,
-					StartTimestamp:      metav1.NewTime(now),
-					CompletionTimestamp: metav1.NewTime(now),
-					Expiration:          metav1.NewTime(now),
+					StartTimestamp:      &timestamp,
+					CompletionTimestamp: &timestamp,
+					Expiration:          &timestamp,
 				},
 			},
 		},
@@ -436,9 +437,9 @@ func TestProcessBackupCompletions(t *testing.T) {
 				Status: velerov1api.BackupStatus{
 					Phase:               velerov1api.BackupPhaseCompleted,
 					Version:             1,
-					Expiration:          metav1.NewTime(now.Add(10 * time.Minute)),
-					StartTimestamp:      metav1.NewTime(now),
-					CompletionTimestamp: metav1.NewTime(now),
+					Expiration:          &metav1.Time{now.Add(10 * time.Minute)},
+					StartTimestamp:      &timestamp,
+					CompletionTimestamp: &timestamp,
 				},
 			},
 		},
@@ -465,9 +466,9 @@ func TestProcessBackupCompletions(t *testing.T) {
 				Status: velerov1api.BackupStatus{
 					Phase:               velerov1api.BackupPhaseCompleted,
 					Version:             1,
-					StartTimestamp:      metav1.NewTime(now),
-					CompletionTimestamp: metav1.NewTime(now),
-					Expiration:          metav1.NewTime(now),
+					StartTimestamp:      &timestamp,
+					CompletionTimestamp: &timestamp,
+					Expiration:          &timestamp,
 				},
 			},
 		},
@@ -496,9 +497,9 @@ func TestProcessBackupCompletions(t *testing.T) {
 				Status: velerov1api.BackupStatus{
 					Phase:               velerov1api.BackupPhaseFailed,
 					Version:             1,
-					StartTimestamp:      metav1.NewTime(now),
-					CompletionTimestamp: metav1.NewTime(now),
-					Expiration:          metav1.NewTime(now),
+					StartTimestamp:      &timestamp,
+					CompletionTimestamp: &timestamp,
+					Expiration:          &timestamp,
 				},
 			},
 		},
@@ -525,9 +526,9 @@ func TestProcessBackupCompletions(t *testing.T) {
 				Status: velerov1api.BackupStatus{
 					Phase:               velerov1api.BackupPhaseFailed,
 					Version:             1,
-					StartTimestamp:      metav1.NewTime(now),
-					CompletionTimestamp: metav1.NewTime(now),
-					Expiration:          metav1.NewTime(now),
+					StartTimestamp:      &timestamp,
+					CompletionTimestamp: &timestamp,
+					Expiration:          &timestamp,
 				},
 			},
 		},
